import os
import sys

coroapi = None

search_paths = os.popen('perl -MConfig -e \'print $Config{sitearch}.",".join(",", @INC);\'').read().rstrip().split(',')
for p in search_paths:
    if os.path.exists(p + '/Coro/CoroAPI.h'):
        coroapi = p

if not coroapi:
    print("unable to find the Coro perl module !!!")
    sys.exit(1)

NAME='coroae'
CFLAGS = os.popen('perl -MExtUtils::Embed -e ccopts').read().rstrip().split()
CFLAGS += ['-Wno-int-to-pointer-cast', '-Wno-error=format', '-Wno-error=int-to-pointer-cast', '-I%s/Coro' % coroapi]
LDFLAGS = []
LIBS = []

GCC_LIST = ['coroae']



import time
import uwsgi
def application(e,s):
    print "locking"
    uwsgi.lock()
    print "locked"
    time.sleep(3)
    uwsgi.unlock()
    print "UN-locked"
    s('200 OK', [('Content-Type','text/html')])
    return "slow"




import os
import subprocess

NAME = 'mono'

CFLAGS = os.popen('pkg-config --cflags mono-2').read().rstrip().split()
LDFLAGS = []
LIBS = os.popen('pkg-config --libs mono-2').read().rstrip().split() 
GCC_LIST = ['mono_plugin']

if os.uname()[0] == 'Darwin':
    LIBS.append('-framework Foundation')

def post_build(config):
    if subprocess.call("sn -k plugins/mono/uwsgi.key", shell=True) != 0:
        os._exit(1)
    if subprocess.call("mcs /target:library /r:System.Web.dll /keyfile:plugins/mono/uwsgi.key plugins/mono/uwsgi.cs", shell=True) != 0:
        os._exit(1)
    print("*** uwsgi.dll available in %s/plugins/mono/uwsgi.dll ***" % os.getcwd())



import uwsgi

def hello():
    pass

def application(env, start_response):
    try:
        uwsgi.register_rpc("A"*300, hello)
        start_response('500 Buffer Overflow', [('Content-Type', 'text/plain')])
    except ValueError:
        start_response('200 OK', [('Content-Type', 'text/plain')])

    return ()



import os
import subprocess

NAME = 'gccgo'

CFLAGS = ['-g']
LDFLAGS = []
LIBS = ['-lgo']
GCC_LIST = ['gccgo_plugin', 'uwsgi.go']

def post_build(config):
    if os.path.exists('plugins/gccgo/uwsgi.go.o'):
        if subprocess.call("objcopy -j .go_export plugins/gccgo/uwsgi.go.o plugins/gccgo/uwsgi.gox", shell=True) != 0:
            os._exit(1)
        print("*** uwsgi.gox available in %s/plugins/gccgo ***" % os.getcwd())



import uwsgi
import time

def slow_task(args):
    time.sleep(10)
    return uwsgi.SPOOL_OK

uwsgi.spooler = slow_task



def application(env, start_response):

    name = uwsgi.spool({'Hello':'World', 'I am a':'long running task'})
    print("spooled as %s" % name)

    start_response('200 Ok', [('Content-Type','text/plain'),('uWSGI-Status', 'spooled')])

    return "task spooled"



# https://github.com/unbit/uwsgi/pull/2615
# CPU heavy application in multi threaded uWSGI doesn't shutdown gracefully.
#
# $ ./uwsgi \
#     --wsgi-file=threads_heavy.py --master --http-socket=:8000 \
#     --workers=4 --threads=8 --max-requests=20 --min-worker-lifetime=6 -L  \
#     --worker-reload-mercy=20 2>&1 | tee uwsgi.log
#
# $ hey -c 16 -z 3m 'http://127.0.0.1:8000/'
#
# $ grep MERCY uwsgi.log
# Tue Mar 19 14:01:59 2024 - worker 1 (pid: 62113) is taking too much time to die...NO MERCY !!!
# Tue Mar 19 14:02:23 2024 - worker 2 (pid: 62218) is taking too much time to die...NO MERCY !!!
# ...
#
# This was caused by pthread_cancel() is called from non-main thread.

def fibonacci(n):
    if n <= 1:
        return n
    return fibonacci(n - 1) + fibonacci(n - 2)


def application(env, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])
    n = 24
    r = fibonacci(n)
    s = f"F({n}) = {r}"
    return [s.encode()]
